from accession.helpers import (
    LruCache,
    PreferredDefaultFilePatch,
    flatten,
    impersonate_file,
    string_to_number,
)


def test_lru_cache_get_has_key_should_reorder():
    cache = LruCache()
    cache.insert("foo", "bar")
    cache.insert("baz", "qux")
    result = cache.get("foo")
    assert result == "bar"
    assert list(cache.data.keys()) == ["baz", "foo"]


def test_lru_cache_get_does_not_have_key_returns_none():
    cache = LruCache()
    cache.insert("foo", "bar")
    result = cache.get("baz")
    assert result is None


def test_lru_cache_insert():
    cache = LruCache(max_size=2)
    cache.insert("foo", "bar")
    cache.insert("baz", "qux")
    cache.insert("spam", "eggs")
    assert cache.get("foo") is None


def test_lru_cache_insert_extant_key_updates_value():
    cache = LruCache(max_size=2)
    cache.insert("foo", "bar")
    cache.insert("baz", "qux")
    cache.insert("foo", "eggs")
    assert cache.get("foo") == "eggs"
    assert list(cache.data.keys()) == ["baz", "foo"]


def test_lru_cache_invalidate_key_in_cache():
    cache = LruCache()
    cache.insert("foo", "bar")
    cache.invalidate("foo")
    assert "foo" not in cache.data.keys()


def test_lru_cache_invalidate_key_not_in_cache_does_not_raise():
    cache = LruCache()
    cache.insert("foo", "bar")
    cache.invalidate("baz")
    assert "foo" in cache.data.keys()


def test_preferred_default_file_patch_eq():
    patch1 = PreferredDefaultFilePatch(at_id="foo", qc_value=3)
    patch2 = PreferredDefaultFilePatch(at_id="foo", qc_value=3)
    assert patch1 == patch2


def test_preferred_default_file_patch_eq_not_equal_when_at_ids_different():
    patch1 = PreferredDefaultFilePatch(at_id="foo", qc_value=3.0)
    patch2 = PreferredDefaultFilePatch(at_id="bar", qc_value=3.0)
    assert patch1 != patch2


def test_preferred_default_file_patch_eq_not_equal_when_qc_values_different():
    patch1 = PreferredDefaultFilePatch(at_id="foo", qc_value=3.0)
    patch2 = PreferredDefaultFilePatch(at_id="foo", qc_value=4.0)
    assert patch1 != patch2


def test_preferred_default_file_patch_get_portal_patch():
    patch = PreferredDefaultFilePatch(at_id="foo", qc_value=3)
    result = patch.get_portal_patch()
    assert result == {"_profile": "file", "_enc_id": "foo", "preferred_default": True}


def test_string_to_number_not_string_return_input():
    not_string = 3
    result = string_to_number(not_string)
    assert result == not_string


def test_string_to_number_stringy_int_returns_int():
    int_string = "3"
    result = string_to_number(int_string)
    assert isinstance(result, int)
    assert result == 3


def test_string_to_number_stringy_float_returns_float():
    float_string = "3.0"
    result = string_to_number(float_string)
    assert isinstance(result, float)
    assert result == 3.0


def test_string_to_number_non_number_string_returns_input():
    non_number_string = "3.0a"
    result = string_to_number(non_number_string)
    assert result == non_number_string


def test_flatten() -> None:
    result = flatten([["a", "b"], ["c", "d"]])
    assert result == ["a", "b", "c", "d"]


def test_impersonate_file() -> None:
    bytes_data = b"firstline\nsecondline\n"
    with impersonate_file(bytes_data) as totally_a_file:
        with open(totally_a_file) as fp:
            data = fp.readlines()
    assert len(data) == 2
    assert isinstance(data[0], str)
    assert data[1] == "secondline\n"



import pytest

from accession.task import Task


@pytest.mark.filesystem
def test_task_init(metadata_json):  # noqa: F811
    task = metadata_json["calls"]["atac.bam2ta"][0]
    new_task = Task("bam2ta", task)
    assert new_task.inputs == task["inputs"]
    assert new_task.outputs == task["outputs"]
    assert new_task.task_name == "bam2ta"
    assert new_task.docker_image == task["dockerImageUsed"]



from logging import FileHandler, StreamHandler
from unittest.mock import mock_open

from accession.logger_factory import logger_factory


def test_logger_factory(mocker):
    mocker.patch("builtins.open", mock_open())
    result = logger_factory(logger_name="foo", log_file_path="bar", no_log_file=False)
    handler_types = [type(hdlr) for hdlr in result.handlers]
    assert len(handler_types) == 2
    assert StreamHandler in handler_types
    assert FileHandler in handler_types


def test_logger_factory_no_log_file():
    """
    Must use a different logger name to avoid conflict with other test.
    """
    result = logger_factory(logger_name="baz", log_file_path="bar", no_log_file=True)
    handler_types = [type(hdlr) for hdlr in result.handlers]
    assert len(handler_types) == 1
    assert StreamHandler in handler_types
    assert FileHandler not in handler_types



import pytest

from accession.backends import AwsBackend, GCBackend, LocalBackend, backend_factory


@pytest.mark.parametrize("uri,expected", [("gs://foo/bar", True), ("not/valid", False)])
def test_gc_backend_is_valid_uri(uri, expected):
    backend = GCBackend()
    result = backend.is_valid_uri(uri)
    assert result is expected


@pytest.mark.parametrize(
    "uri,expected", [("s3://foo/bar", True), ("/not/valid", False)]
)
def test_aws_backend_is_valid_uri(uri, expected):
    backend = AwsBackend()
    result = backend.is_valid_uri(uri)
    assert result is expected


def test_local_backend_is_valid_uri():
    backend = LocalBackend()
    result = backend.is_valid_uri("foo")
    assert result is True


@pytest.mark.parametrize(
    "backend_name,expected_type",
    [
        ("Local", LocalBackend),
        ("sge", LocalBackend),
        ("aws", AwsBackend),
        ("gcp", GCBackend),
    ],
)
def test_backend_factory(backend_name, expected_type):
    result = backend_factory(backend_name)
    assert isinstance(result, expected_type)


def test_backend_factory_invalid_backend_name_raises():
    with pytest.raises(ValueError):
        backend_factory("invalid")
